@@ -9,11 +9,15 @@ import time
 import hashlib
 import fnmatch
 
-from enebootools.mergetool import flpatchqs, flpatchlxml, flpatchpy
+from enebootools.mergetool import flpatchqs, flpatchlxml, flpatchpy, flpatchapipy
 
 
-def filepath(): return os.path.abspath(os.path.dirname(__file__))
-def filedir(x): return os.path.abspath(os.path.join(filepath(), x))
+def filepath():
+    return os.path.abspath(os.path.dirname(__file__))
+
+
+def filedir(x):
+    return os.path.abspath(os.path.join(filepath(), x))
 
 
 def hash_file(dirname, filename):
@@ -35,16 +39,16 @@ def hash_none():
 def _xf(x, cstring=False, **kwargs):  # xml-format
     if type(x) is list:
         return "\n---\n\n".join([_xf(x1) for x1 in x])
-    if 'encoding' not in kwargs:
-        kwargs['encoding'] = "UTF8"
-    if 'pretty_print' not in kwargs:
-        kwargs['pretty_print'] = True
+    if "encoding" not in kwargs:
+        kwargs["encoding"] = "UTF8"
+    if "pretty_print" not in kwargs:
+        kwargs["pretty_print"] = True
 
     value = etree.tostring(x, **kwargs)
     if cstring:
         return value
     else:
-        return str(value, kwargs['encoding'])
+        return str(value, kwargs["encoding"])
 
 
 class FolderApplyPatch(object):
@@ -53,8 +57,7 @@ class FolderApplyPatch(object):
         if patchdir[-1] == "/":
             patchdir = patchdir[:-1]
         try:
-            patchname = open(os.path.join(patchdir, "conf",
-                             "patch_series")).read().strip()
+            patchname = open(os.path.join(patchdir, "conf", "patch_series")).read().strip()
             newpatchdir = os.path.join(patchdir, "patches", patchname)
             iface.warn("Cambiando carpeta de parche a %s" % newpatchdir)
             patchdir = newpatchdir
@@ -72,7 +75,8 @@ class FolderApplyPatch(object):
                 break
         if self.patch_dir is None:
             self.iface.error(
-                "No pude encontrar %s en ninguna subcarpeta del parche." % expected_file)
+                "No pude encontrar %s en ninguna subcarpeta del parche." % expected_file
+            )
             self.patch_dir = patchdir
 
         patch_file = os.path.join(self.patch_dir, expected_file)
@@ -95,7 +99,6 @@ class FolderApplyPatch(object):
         if self.root is None:
             return
         for action in self.root:
-
             actionname = action.tag
             if not isinstance(actionname, str):
                 continue
@@ -119,14 +122,14 @@ class FolderApplyPatch(object):
                     self.patch_xml(action, folder)
                 elif actionname == "patchpy":
                     self.patch_py(action, folder)
-                    
+
                 # TODO: actionname == "patchphp"
                 else:
-                    self.iface.warn(
-                        "** Se ha ignorado acción desconocida %s **" % repr(actionname))
+                    self.iface.warn("** Se ha ignorado acción desconocida %s **" % repr(actionname))
             except Exception as e:
                 self.iface.exception(
-                    "ComputePatch", "No se pudo aplicar el parche para %s" % action.get("name"))
+                    "ComputePatch", "No se pudo aplicar el parche para %s" % action.get("name")
+                )
 
             tend = time.time()
             tdelta = tend - tbegin
@@ -167,7 +170,8 @@ class FolderApplyPatch(object):
         if not os.path.exists(os.path.join(folder, module_path)):
             if os.path.relpath(path, module_path).count("/") > 0:
                 self.iface.warn(
-                    "Ignorando la creación de fichero %s (el módulo no existe)" % filename)
+                    "Ignorando la creación de fichero %s (el módulo no existe)" % filename
+                )
                 return
 
         pathname = os.path.join(path, filename)
@@ -188,8 +192,7 @@ class FolderApplyPatch(object):
         while module_path.count("/") > 1:
             module_path = os.path.dirname(module_path)
         if not os.path.exists(os.path.join(folder, module_path)):
-            self.iface.info(
-                "Ignorando el borrado de fichero %s (el módulo no existe)" % filename)
+            self.iface.info("Ignorando el borrado de fichero %s (el módulo no existe)" % filename)
             return
 
         pathname = os.path.join(path, filename)
@@ -202,8 +205,7 @@ class FolderApplyPatch(object):
                 self.iface.info("Borrando %s . . ." % filename)
                 os.unlink(dst)
             else:
-                self.iface.warn(
-                    "Se iba a borrar %s, pero no existe." % filename)
+                self.iface.warn("Se iba a borrar %s, pero no existe." % filename)
 
     def replace_file(self, replacefile, folder):
         path = replacefile.get("path")
@@ -213,7 +215,8 @@ class FolderApplyPatch(object):
         dst = os.path.join(folder, pathname)
         if not os.path.exists(dst):
             self.iface.warn(
-                "Ignorando reemplazo de fichero para %s (el fichero no existe)" % filename)
+                "Ignorando reemplazo de fichero para %s (el fichero no existe)" % filename
+            )
             return
 
         self.iface.debug("Reemplazando fichero %s . . ." % filename)
@@ -231,8 +234,7 @@ class FolderApplyPatch(object):
         dst = os.path.join(folder, pathname)
 
         if not os.path.exists(dst):
-            self.iface.warn(
-                "Ignorando parche QS para %s (el fichero no existe)" % filename)
+            self.iface.debug("Ignorando parche QS para %s (el fichero no existe)" % filename)
             return
         self.iface.info("Aplicando parche QS %s . . ." % filename)
         old_output = self.iface.output
@@ -241,10 +243,10 @@ class FolderApplyPatch(object):
         if self.iface.verbosity < 0:
             self.iface.verbosity = 0
         old_style, self.iface.patch_qs_style_name = self.iface.patch_qs_style_name, style
-        self.iface.set_output_file(dst+".patched")
-        if style in ['legacy']:
+        self.iface.set_output_file(dst + ".patched")
+        if style in ["legacy"]:
             ret = flpatchqs.patch_qs(self.iface, dst, src)
-        elif style in ['qsdir']:
+        elif style in ["qsdir"]:
             ret = flpatchqs.patch_qs_dir(self.iface, dst, src)
         else:
             raise ValueError("Estilo de parche QS desconocido: %s" % style)
@@ -252,15 +254,14 @@ class FolderApplyPatch(object):
         self.iface.verbosity = old_verbosity
         self.iface.patch_qs_style_name = old_style
         if not ret:
-            self.iface.warn(
-                "Hubo algún problema aplicando el parche QS para %s" % filename)
+            self.iface.warn("Hubo algún problema aplicando el parche QS para %s" % filename)
             try:
-                os.unlink(dst+".patched")
+                os.unlink(dst + ".patched")
             except IOError:
                 pass
         else:
             os.unlink(dst)
-            os.rename(dst+".patched", dst)
+            os.rename(dst + ".patched", dst)
 
     def patch_xml(self, patchxml, folder):
         style = patchxml.get("style", "legacy1")
@@ -272,8 +273,7 @@ class FolderApplyPatch(object):
         dst = os.path.join(folder, pathname)
 
         if not os.path.exists(dst):
-            self.iface.warn(
-                "Ignorando parche XML para %s (el fichero no existe)" % filename)
+            self.iface.debug("Ignorando parche XML para %s (el fichero no existe)" % filename)
             return
         self.iface.info("Aplicando parche XML %s . . ." % filename)
         old_output = self.iface.output
@@ -281,21 +281,20 @@ class FolderApplyPatch(object):
         self.iface.verbosity -= 2
         if self.iface.verbosity < 0:
             self.iface.verbosity = min([0, self.iface.verbosity])
-        self.iface.set_output_file(dst+".patched")
+        self.iface.set_output_file(dst + ".patched")
         ret = flpatchlxml.patch_lxml(self.iface, src, dst)
         self.iface.output = old_output
         self.iface.verbosity = old_verbosity
         if not ret:
-            self.iface.warn(
-                "Hubo algún problema aplicando el parche XML para %s" % filename)
+            self.iface.warn("Hubo algún problema aplicando el parche XML para %s" % filename)
             try:
-                os.unlink(dst+".patched")
+                os.unlink(dst + ".patched")
             except IOError:
                 pass
         else:
             os.unlink(dst)
-            os.rename(dst+".patched", dst)
-    
+            os.rename(dst + ".patched", dst)
+
     def patch_py(self, patchscript, folder):
         style = patchscript.get("style", "legacy")
         path = patchscript.get("path")
@@ -306,8 +305,7 @@ class FolderApplyPatch(object):
         dst = os.path.join(folder, pathname)
 
         if not os.path.exists(dst):
-            self.iface.warn(
-                "Ignorando parche PY para %s (el fichero no existe)" % filename)
+            self.iface.debug("Ignorando parche PY para %s (el fichero no existe)" % filename)
             return
         self.iface.info("Aplicando parche PY %s . . ." % filename)
         old_output = self.iface.output
@@ -316,10 +314,13 @@ class FolderApplyPatch(object):
         if self.iface.verbosity < 0:
             self.iface.verbosity = 0
         old_style, self.iface.patch_py_style_name = self.iface.patch_py_style_name, style
-        self.iface.set_output_file(dst+".patched")
-        if style in ['legacy']:
-            ret = flpatchpy.patch_py(self.iface, dst, src)
-        elif style in ['qsdir']:
+        self.iface.set_output_file(dst + ".patched")
+        if style in ["legacy"]:
+            if filename.endswith(("_api.py", "_schema.py", "_model.py")):
+                ret = flpatchapipy.patch_py(self.iface, dst, src)
+            else:
+                ret = flpatchpy.patch_py(self.iface, dst, src)
+        elif style in ["qsdir"]:
             ret = flpatchpy.patch_py_dir(self.iface, dst, src)
         else:
             raise ValueError("Estilo de parche PY desconocido: %s" % style)
@@ -327,20 +328,19 @@ class FolderApplyPatch(object):
         self.iface.verbosity = old_verbosity
         self.iface.patch_qs_style_name = old_style
         if not ret:
-            self.iface.warn(
-                "Hubo algún problema aplicando el parche PY para %s" % filename)
+            self.iface.warn("Hubo algún problema aplicando el parche PY para %s" % filename)
             try:
-                os.unlink(dst+".patched")
+                os.unlink(dst + ".patched")
             except IOError:
                 pass
         else:
             os.unlink(dst)
-            os.rename(dst+".patched", dst)
+            os.rename(dst + ".patched", dst)
 
 
 class FolderCreatePatch(object):
     nsmap = {
-        'flpatch': "http://www.abanqg2.com/es/directori/abanq-ensambla/?flpatch",
+        "flpatch": "http://www.abanqg2.com/es/directori/abanq-ensambla/?flpatch",
     }
 
     def __init__(self, iface, basedir, finaldir, patchdir):
@@ -361,7 +361,8 @@ class FolderCreatePatch(object):
         self.encoding = "iso-8859-15"
         # <flpatch:modifications name="patchname" >
         self.root = etree.Element(
-            "{%s}modifications" % self.nsmap['flpatch'], name=self.patch_name, nsmap=self.nsmap)
+            "{%s}modifications" % self.nsmap["flpatch"], name=self.patch_name, nsmap=self.nsmap
+        )
         self.tree = self.root.getroottree()
         ignored_files = [
             "*~",
@@ -409,21 +410,17 @@ class FolderCreatePatch(object):
         self.added_files = finaldir_files - basedir_files
         self.deleted_files = basedir_files - finaldir_files
         self.common_files = finaldir_files & basedir_files
-        # print "+" , self.added_files
-        # print "-" , self.deleted_files
-        # print "=" , self.common_files
+        # iface.info("+ %s" % self.added_files)
+        # iface.info("- %s" % self.deleted_files)
+        # print("=", self.common_files)
 
         if basedir and finaldir:
-
             iface.info("Calculando diferencias . . . ")
 
             file_actions = []
-            file_actions += [(os.path.dirname(f), f, "add")
-                             for f in self.added_files]
-            file_actions += [(os.path.dirname(f), f, "common")
-                             for f in self.common_files]
-            file_actions += [(os.path.dirname(f), f, "delete")
-                             for f in self.deleted_files]
+            file_actions += [(os.path.dirname(f), f, "add") for f in self.added_files]
+            file_actions += [(os.path.dirname(f), f, "common") for f in self.common_files]
+            file_actions += [(os.path.dirname(f), f, "delete") for f in self.deleted_files]
             # Intentar guardarlos de forma ordenada, para minimizar las diferencias entre parches.
             for path, filename, action in sorted(file_actions):
                 if action == "add":
@@ -439,8 +436,9 @@ class FolderCreatePatch(object):
         path, name = os.path.split(filename)
         if len(path) and not path.endswith("/"):
             path += "/"
-        newnode = etree.SubElement(self.root, "{%s}%s" % (
-            self.nsmap['flpatch'], actionname), path=path, name=name)
+        newnode = etree.SubElement(
+            self.root, "{%s}%s" % (self.nsmap["flpatch"], actionname), path=path, name=name
+        )
         return newnode
 
     def add_file(self, filename):
@@ -512,16 +510,20 @@ class FolderCreatePatch(object):
                     # TODO: actionname == "patchphp"
                     else:
                         self.iface.warn(
-                            "** Se ha ignorado acción desconocida %s **" % repr(actionname))
+                            "** Se ha ignorado acción desconocida %s **" % repr(actionname)
+                        )
                 else:
                     if actionname == "patchxml":
                         action.set("style", self.iface.patch_xml_style_name)
                     else:
                         self.iface.warn(
-                            "** Se ha ignorado el setStype de acción desconocida %s **" % repr(actionname))
+                            "** Se ha ignorado el setStype de acción desconocida %s **"
+                            % repr(actionname)
+                        )
             except Exception as e:
                 self.iface.exception(
-                    "ComputePatch", "No se pudo computar el parche para %s" % action.get("name"))
+                    "ComputePatch", "No se pudo computar el parche para %s" % action.get("name")
+                )
 
             if ret == -1:
                 self.root.remove(action)
@@ -531,8 +533,7 @@ class FolderCreatePatch(object):
                 self.iface.debug("La operación tomó %.2f segundos" % tdelta)
 
         f1 = open(self.patch_filename, "wb")
-        f1.write(_xf(self.root, xml_declaration=False,
-                 cstring=True, encoding=self.encoding))
+        f1.write(_xf(self.root, xml_declaration=False, cstring=True, encoding=self.encoding))
         f1.close()
 
     def compute_delete_file(self, addfile):
@@ -581,21 +582,21 @@ class FolderCreatePatch(object):
         if self.iface.verbosity < 0:
             self.iface.verbosity = min([0, self.iface.verbosity])
         self.iface.set_output_file(dst)
-        if self.iface.patch_qs_style_name in ['legacy']:
+        if self.iface.patch_qs_style_name in ["legacy"]:
             ret = flpatchqs.diff_qs(self.iface, base, final)
-        elif self.iface.patch_qs_style_name in ['qsdir']:
+        elif self.iface.patch_qs_style_name in ["qsdir"]:
             ret = flpatchqs.diff_qs_dir(self.iface, base, final)
         else:
-            raise ValueError("patch_qs_style_name no reconocido: %s" %
-                             self.iface.patch_qs_style_name)
+            raise ValueError(
+                "patch_qs_style_name no reconocido: %s" % self.iface.patch_qs_style_name
+            )
         self.iface.output = old_output
         self.iface.verbosity = old_verbosity
         if ret == -1:
             os.unlink(dst)
             return -1
         if not ret:
-            self.iface.warn(
-                "Pudo haber algún problema generando el parche QS para %s" % filename)
+            self.iface.warn("Pudo haber algún problema generando el parche QS para %s" % filename)
 
     def compute_patch_xml(self, patchxml):
         patchxml.set("style", self.iface.patch_xml_style_name)
@@ -619,13 +620,11 @@ class FolderCreatePatch(object):
             os.unlink(dst)
             return -1
         if not ret:
-            self.iface.warn(
-                "Pudo haber algún problema generando el parche XML para %s" % filename)
+            self.iface.warn("Pudo haber algún problema generando el parche XML para %s" % filename)
 
 
 def diff_folder(iface, basedir, finaldir, patchdir, inplace=False):
-    iface.debug("Folder Diff $basedir:%s $finaldir:%s $patchdir:%s" %
-                (basedir, finaldir, patchdir))
+    iface.debug("Folder Diff $basedir:%s $finaldir:%s $patchdir:%s" % (basedir, finaldir, patchdir))
     # patchdir no debe existir
     parent_patchdir = os.path.abspath(os.path.join(patchdir, ".."))
     if not os.path.exists(parent_patchdir):
@@ -639,8 +638,7 @@ def diff_folder(iface, basedir, finaldir, patchdir, inplace=False):
         return
     if not inplace:
         if os.path.lexists(patchdir):
-            iface.error(
-                "La ruta a $finaldir %s ya existía. No se continua. " % patchdir)
+            iface.error("La ruta a $finaldir %s ya existía. No se continua. " % patchdir)
             return
     if not os.path.lexists(patchdir):
         os.mkdir(patchdir)
@@ -650,8 +648,9 @@ def diff_folder(iface, basedir, finaldir, patchdir, inplace=False):
 
 
 def patch_folder(iface, basedir, finaldir, patchdir):
-    iface.debug("Folder Patch $basedir:%s $finaldir:%s $patchdir:%s" %
-                (basedir, finaldir, patchdir))
+    iface.debug(
+        "Folder Patch $basedir:%s $finaldir:%s $patchdir:%s" % (basedir, finaldir, patchdir)
+    )
     if not os.path.exists(basedir):
         iface.error("La ruta %s no existe" % basedir)
         return
@@ -673,8 +672,7 @@ def patch_folder(iface, basedir, finaldir, patchdir):
             iface.error("La ruta %s no existe" % parent_finaldir)
             return
         if os.path.lexists(finaldir):
-            iface.error(
-                "La ruta a $finaldir %s ya existía. No se continua. " % finaldir)
+            iface.error("La ruta a $finaldir %s ya existía. No se continua. " % finaldir)
             return
 
         os.mkdir(finaldir)
@@ -693,6 +691,140 @@ def patch_folder(iface, basedir, finaldir, patchdir):
     fpatch.patch_folder(finaldir)
 
 
+def update_patch_folder(iface, finaldir, srcdir, patchdir, path):
+    basedir = os.path.join(path, "build/base")
+    mod_files = []
+
+    fpatch = FolderCreatePatch(iface, finaldir, srcdir, patchdir)
+    for action in fpatch.root:
+        src_file = os.path.join(srcdir, action.get("path"), action.get("name"))
+        final_file = os.path.join(finaldir, action.get("path"), action.get("name"))
+        base_file = os.path.join(basedir, action.get("path"), action.get("name"))
+        src_time = os.path.getmtime(src_file) if os.path.exists(src_file) else 0
+        final_time = os.path.getmtime(final_file) if os.path.exists(final_file) else 0
+        if src_time and final_time and src_time <= final_time:
+            fpatch.root.remove(action)
+            continue
+
+        if str(action.tag).lower().endswith(
+            ("patchscript", "patchxml", "pathpy")
+        ) and not os.path.exists(os.path.join(basedir, action.get("path"), action.get("name"))):
+            action.tag = "{http://www.abanqg2.com/es/directori/abanq-ensambla/?flpatch}addFile"
+        # if str(action.tag).endswith("deleteFile"):
+        #    fpatch.root.remove(action)
+
+        mod_files.append([action.get("path"), action.get("name")])
+
+    iface.info("Actualizando ficheros entre %s y %s" % (basedir, srcdir))
+    for mod_file in mod_files:
+        update_patch_file(iface, mod_file, patchdir, basedir, srcdir, finaldir)
+
+    iface.info("Changes : %s" % mod_files)
+
+    update_xml_patch(iface, fpatch, basedir)
+
+    iface.info("Listo!")
+
+
+def update_patch_file(iface, mod_file, patchdir, basedir, srcdir, finaldir):
+    file_name = str(mod_file[1])
+    base_file = os.path.join(basedir, *mod_file)
+    final_file = os.path.join(finaldir, *mod_file)
+    src_file = os.path.join(srcdir, *mod_file)
+
+    ext = "XML"
+    if file_name.upper().endswith("QS"):
+        ext = "QS"
+    elif file_name.upper().endswith("PY"):
+        ext = "PY"
+    patch_file = os.path.join(patchdir, file_name)
+
+    iface.set_output_file(patch_file)
+
+    if (
+        os.path.exists(final_file) and os.path.exists(src_file) and os.path.exists(base_file)
+    ):  # Si existe en base , final y src Update
+        iface.info("Update file %s -> %s, patch: %s" % (src_file, base_file, patch_file))
+        return iface.do_file_diff(ext, base_file, src_file)
+    elif not os.path.exists(base_file) and os.path.exists(
+        src_file
+    ):  # Si no existe en base y si en src es nuevo!
+        iface.info("New file %s -> %s, patch: %s" % (src_file, base_file, patch_file))
+        shutil.copyfile(src_file, final_file)
+        with open(src_file, "rb") as file_:
+            iface.output.write(file_.read())
+    elif os.path.exists(base_file) and not os.path.exists(
+        src_file
+    ):  # Si no existe en base y si en src, es delete!
+        iface.info("Delete file %s -> %s, patch: %s" % (src_file, final_file, patch_file))
+        os.remove(final_file)
+
+    return True
+
+
+def update_xml_patch(iface, fpatch, basedir):
+    patch_xml_file = os.path.join(fpatch.patchdir, fpatch.patch_name + ".xml")
+    iface.info("Calculando cambios en %s" % patch_xml_file)
+
+    current_et = etree.parse(patch_xml_file)
+    current_root = current_et.getroot()
+
+    found_changes = False
+
+    for action in fpatch.root:
+        found_changes = True
+        new_path = action.get("path")
+        new_name = str(action.get("name"))
+        if new_name.endswith("qs"):
+            action.set("style", iface.patch_qs_style_name)
+        elif new_name.endswith("py"):
+            action.set("style", iface.patch_py_style_name)
+        else:
+            action.set("style", iface.patch_xml_style_name)
+
+        new_action = str(action.tag).split("}")[1]
+        found = False
+        for current_action in current_root:
+            current_path = current_action.get("path")
+            current_name = current_action.get("name")
+            if current_path == new_path and current_name == new_name:
+                current_root.remove(current_action)
+                iface.info("Borrando %s %s" % (current_path, current_name))
+                del current_action
+                found = True
+                break
+
+        iface.info(
+            "%s linea %s %s"
+            % ("Editando" if found else "Nueva", new_action, os.path.join(new_path, new_name))
+        )
+
+        if new_action == "deleteFile":
+            full_file_path = os.path.join(fpatch.patchdir, new_name)
+            if os.path.exists(full_file_path):
+                os.remove(full_file_path)
+
+            full_file_base = os.path.join(basedir, current_path, current_name)
+            if not os.path.exists(full_file_base):
+                continue
+
+        current_root.append(action)
+
+    if not found_changes:
+        iface.info("No hay cambios!")
+        return
+    iface.info("Cambios detectados!")
+    iface.info("Guardando cambios en %s" % patch_xml_file)
+
+    file_ = open(patch_xml_file, "w", encoding="UTF-8")
+    result = _xf(current_et)
+    result = result.replace("/></flpatch", "/>\n</flpatch")
+    result = result.replace("\n<flpatch", "\n  <flpatch")
+    result = result.replace("/><flpatch", "/>\n  <flpatch")
+    file_.write(result)
+    file_.close()
+
+
 def patch_folder_inplace(iface, patchdir, finaldir):
     fpatch = FolderApplyPatch(iface, patchdir)
     fpatch.patch_folder(finaldir)
