@@ -1,5 +1,5 @@
 # encoding: UTF-8
-import zlib, os, sys, re, hashlib
+import zlib, os, sys, re, hashlib, pathlib
 from enebootools.packager.pkgsplitter import to_uint32
 from enebootools.packager import __version__, __PROGRAM__NAME__
 from enebootools.lib.utils import find_files
@@ -161,17 +161,23 @@ def createpkg(iface, modulefolder, dst_file, emulate_mode):
         list_modules.append(modulename)
         for filename in files:
             bname, ext = os.path.splitext(filename)
-
-            if bname.startswith("test") and not iface.include_test:
-                # print("Ignorando %s" % bname)
-                continue
-
+            
             if ext not in load_ext:
                 ignored_ext.add(ext)
                 continue
 
+            filepath = os.path.abspath(os.path.join(fpath, filename))
+            path_dirs_list = pathlib.Path(filepath)
+            if "test" in path_dirs_list.parts and not iface.include_test:
+                print("fichero %s incluye carpeta 'test' en path. Ignorado." % (filepath))
+                continue
+
+            if os.path.basename(filename).startswith("test_") and not iface.include_test:
+                print("fichero %s comienza por 'test_'. Ignorado." % (filepath))
+                continue
+
             file_basename = os.path.basename(filename)
-            filepath = os.path.join(fpath, filename)
+
             sha1text = hashlib.sha1(open(filepath, "rb").read()).hexdigest()
             sha1text = sha1text.upper()
             shasum += sha1text
